
Imports System.Collections.Generic
Imports System.Threading
Imports System.IO
Imports System.Media

Public Class Form_Main
    ' ------------------------------------------------------------------------- PROCESSES
    Private AcquisitionThread As Thread
    Private AcquisitionTimer As Stopwatch = New Stopwatch()
    ' ------------------------------------------------------------------------- ACQUISITION
    Private WriteDataFile As Boolean
    ' ------------------------------------------------------------------------- 
    Private SamplingFreq_hz As Single
    Private SamplingTime_ms As Int32
    Private NumChannels As Int32
    Private NumChannels_Max As Int32 = 25
    Private ZeroingTime_ms As Int32
    Private AcquisitionTime_ms As Int32
    ' ------------------------------------------------------------------------- 
    Private VisualizationFreq_hz As Single
    Private VisualizationTime_ms As Int32 = 2
    Private ZoomX As Int32
    Private ZoomY As Int32
    ' ------------------------------------------------------------------------- 
    Private MidValue(NumChannels_Max - 1) As Single
    Private LastValue(NumChannels_Max - 1) As Single
    Private OldValue(NumChannels_Max - 1) As Single
    ' ------------------------------------------------------------------------- 
    Private AcquisitionSystem As String
    Private TransducerType As String
    Private SurveyName As String
    Private Department As String
    Private Locality As String
    Private Notes As String
    Private StationCode As String
    Private VoltageGain As Int32
    Private OutDataFormat As String
    Private IdChannel1 As String
    Private IdChannel2 As String
    Private IdChannel3 As String
    Private Rotation As String
    ' ------------------------------------------------------------------------- DRAWING
    Private Operation As Int32
    Private Percentual As Int32
    Private UpdatingGraph As Boolean = False
    Private GFX As Graphics
    Private PenColors As Pen() = New Pen(24) {}
    Private x As Int32
    Private ReadSamplesTotal As Integer
    Private StopAcquisitionFlag As Boolean

    ' ===============================================================
    '   MAIN FORM
    ' ===============================================================   
    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        '----------------------------------------------------------------------
        Text = AppTitleAndVersion("Theremino Dolfrang")
        '---------------------------------------------------------------------- LOAD INITIAL PARAMS
        Load_INI()
        SetLocales()
        'ToolTips_Init()
        ToolStrip1.Renderer = New ToolStripButtonRenderer
        ShowKvalues()
        '---------------------------------------------------------------------- START HAL 
        StartThereminoHAL()
        ' --------------------------------------------------------------------- COLORS
        PenColors(0) = New Pen(Brushes.Red, 1)
        PenColors(1) = New Pen(Brushes.Red, 1)
        PenColors(2) = New Pen(Brushes.Green)
        PenColors(3) = New Pen(Brushes.Blue)
        PenColors(4) = New Pen(Brushes.Red)
        PenColors(5) = New Pen(Brushes.Green)
        PenColors(6) = New Pen(Brushes.Blue)
        PenColors(7) = New Pen(Brushes.Red)
        PenColors(8) = New Pen(Brushes.Green)
        PenColors(9) = New Pen(Brushes.Blue)
        PenColors(10) = New Pen(Brushes.Red)
        PenColors(11) = New Pen(Brushes.Green)
        PenColors(12) = New Pen(Brushes.Blue)
        PenColors(13) = New Pen(Brushes.Red)
        PenColors(14) = New Pen(Brushes.Green)
        PenColors(15) = New Pen(Brushes.Blue)
        PenColors(16) = New Pen(Brushes.Red)
        PenColors(17) = New Pen(Brushes.Green)
        PenColors(18) = New Pen(Brushes.Blue)
        PenColors(19) = New Pen(Brushes.Red)
        PenColors(20) = New Pen(Brushes.Green)
        PenColors(21) = New Pen(Brushes.Blue)
        PenColors(22) = New Pen(Brushes.Red)
        PenColors(23) = New Pen(Brushes.Green)
        PenColors(24) = New Pen(Brushes.Blue)
        ' -------------------------------------------------------------------------------
        ' REAL TIME REQUESTING
        ' -------------------------------------------------------------------------------
        System.Threading.Thread.CurrentThread.Priority = System.Threading.ThreadPriority.Normal
        System.Diagnostics.Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.RealTime
        ' -------------------------------------------------------------------------------
        UserInterfaceTimer.Start()
        EventsAreEnabled = True
        ' ------------------------------------------- To be removed in the next version
        TabPage2.Dispose()
        ' ------------------------------------------- FadeIn
        Refresh()
        Forms_FadeTo(1, 400)
    End Sub
    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        Forms_FadeTo(0, 500)
        Me.Refresh()
        Save_INI()
        StopThereminoHAL()
    End Sub
    Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        InitPictureboxImage()
    End Sub
    Private Sub Form_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        If Not EventsAreEnabled Then Exit Sub
        LimitFormPosition(Me)
    End Sub

    ' ===================================================================================
    '  MenuStrip and ToolStrip accepting the first click
    '  If the form receives a WM_PARENTNOTIFY (528) message and is not focused 
    '  then the form is activated before to execute the message
    ' ===================================================================================
    Protected Overrides Sub WndProc(ByRef m As Message)
        If Not Me.Focused AndAlso m.Msg = 528 Then
            Me.Activate()
        End If
        MyBase.WndProc(m)
    End Sub

    ' ===================================================================================
    '   MenuStrip and ToolStrip Gradients
    ' ===================================================================================
    Private Sub MenuStrip1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles MenuStrip1.Paint
        Dim bounds As New Rectangle(0, 0, _
                                    MenuStrip1.Width, MenuStrip1.Height)
        Dim brush As New Drawing2D.LinearGradientBrush(bounds, _
                                                      Color.FromArgb(240, 240, 240), _
                                                      Color.FromArgb(200, 200, 200), _
                                                      Drawing2D.LinearGradientMode.Horizontal)
        e.Graphics.FillRectangle(brush, bounds)
    End Sub
    Private Sub ToolStrip1_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles ToolStrip1.Paint
        Dim bounds As New Rectangle(0, 0, _
                                    ToolStrip1.Width, ToolStrip1.Height)
        Dim brush As New Drawing2D.LinearGradientBrush(bounds, _
                                                       Color.White, _
                                                       Color.FromArgb(200, 200, 200), _
                                                       Drawing2D.LinearGradientMode.Vertical)
        e.Graphics.FillRectangle(brush, bounds)
    End Sub


    ' ===================================================================================
    '   ToolStrip PressedButton color
    ' ===================================================================================
    Class ToolStripButtonRenderer
        Inherits System.Windows.Forms.ToolStripProfessionalRenderer
        Protected Overrides Sub OnRenderButtonBackground(ByVal e As ToolStripItemRenderEventArgs)
            Dim btn As ToolStripButton = CType(e.Item, ToolStripButton)
            If btn IsNot Nothing AndAlso btn.CheckOnClick AndAlso btn.Checked Then
                Dim bounds As Rectangle = New Rectangle(0, 0, e.Item.Width - 1, e.Item.Height - 1)
                Dim brush As New Drawing2D.LinearGradientBrush(bounds, _
                                                               Color.Gold, _
                                                               Color.FromArgb(255, 255, 255), _
                                                               Drawing2D.LinearGradientMode.Vertical)
                e.Graphics.FillRectangle(brush, bounds)
                e.Graphics.DrawRectangle(Pens.Orange, bounds)
            Else
                MyBase.OnRenderButtonBackground(e)
            End If
        End Sub
    End Class


    ' =======================================================================================
    '   MENU
    ' =======================================================================================
    Private Sub Menu_File_Open_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Menu_File_OpenFolder.Click
        Process.Start(Application.StartupPath)
    End Sub
    Private Sub Menu_File_Exit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Menu_File_Exit.Click
        Me.Close()
    End Sub
    Private Sub Menu_Info_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Menu_About.Click
        Form_About.ShowDialog(Me)
    End Sub

    ' =======================================================================================
    '   MENU LANGUAGE
    ' =======================================================================================
    Private Sub Menu_Language_DropDownOpening(ByVal sender As Object, ByVal e As System.EventArgs) Handles Menu_Language.DropDownOpening
        For Each item As ToolStripMenuItem In Menu_Language.DropDownItems
            If item.Name.EndsWith(Language, StringComparison.InvariantCultureIgnoreCase) Then
                item.Select()
            End If
        Next
    End Sub
    Private Sub Menu_Language_Deutsch_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Menu_Language_Deutsch.Click
        Language = "Deutsch"
        SetLocales()
        Save_INI()
        'ToolTips_Init()
    End Sub
    Private Sub Menu_Language_English_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Menu_Language_English.Click
        Language = "English"
        SetLocales()
        Save_INI()
        'ToolTips_Init()
    End Sub
    Private Sub Menu_Language_Espanol_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Menu_Language_Espanol.Click
        Language = "Espanol"
        SetLocales()
        Save_INI()
        'ToolTips_Init()
    End Sub
    Private Sub Menu_Language_Francais_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Menu_Language_Francais.Click
        Language = "Francais"
        SetLocales()
        Save_INI()
        'ToolTips_Init()
    End Sub
    Private Sub Menu_Language_Italian_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Menu_Language_Italian.Click
        Language = "Italian"
        SetLocales()
        Save_INI()
        'ToolTips_Init()
    End Sub
    Private Sub Menu_Language_Japanese_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Menu_Language_Japanese.Click
        Language = "Japanese"
        SetLocales()
        Save_INI()
        'ToolTips_Init()
    End Sub


    ' =======================================================================================
    '   TOOLBAR
    ' =======================================================================================
    Private Sub ToolStripButton_OpenProgramFolder_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButton_OpenFolder.Click
        Process.Start(Application.StartupPath)
        ToolStripButton_OpenFolder.Checked = False
    End Sub
    Private Sub ToolStripButton_StartAcquisition_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButton_Start.Click
        TabControl1.SelectedIndex = 1 ' 2 if trimming page is visible
        Refresh()
        StartAcquisition()
        ToolStripButton_Start.Checked = False
    End Sub
    Private Sub ToolStripButton_StopAcquisition_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripButton_Stop.Click
        StopAcquisitionFlag = True
        ToolStripButton_Stop.Checked = False
    End Sub


    ' =========================================================================
    '   THEREMINO HAL - START STOP -
    ' =========================================================================
    Private HalProcess As Process = Nothing
    Private Sub StartThereminoHAL()
        Dim path As String = Application.StartupPath & "\Theremino_HAL.exe"
        If Not My.Computer.FileSystem.FileExists(path) Then
            path = Application.StartupPath & "\Theremino_HAL\Theremino_HAL.exe"
        End If
        If Not My.Computer.FileSystem.FileExists(path) Then Return
        Dim psi As New ProcessStartInfo
        psi.WorkingDirectory = IO.Path.GetDirectoryName(path)
        psi.FileName = path
        HalProcess = Process.Start(psi)
    End Sub
    Private Sub StopThereminoHAL()
        If HalProcess IsNot Nothing AndAlso Not HalProcess.HasExited Then
            Try
                HalProcess.CloseMainWindow()
            Finally
            End Try
        End If
    End Sub


    ' ===============================================================
    '    MAIN BITMAP
    ' =============================================================== 
    Private Sub InitPictureboxImage()
        With PictureBox1
            If .ClientSize.Height < 1 Then Return
            If .Image Is Nothing OrElse .Image.Width <> .ClientSize.Width _
                                 OrElse .Image.Height <> .ClientSize.Height Then
                .Image = New Bitmap(.ClientRectangle.Width, _
                                    .ClientRectangle.Height, _
                                    Imaging.PixelFormat.Format24bppRgb)
            End If
            GFX = Graphics.FromImage(PictureBox1.Image)
            GFX.Clear(PictureBox1.BackColor)
            .Refresh()
        End With
    End Sub


    ' ===============================================================
    '    WINDOW TIMERS PRECISION
    ' =============================================================== 
    <System.Runtime.InteropServices.DllImport("winmm.dll")> _
    Private Shared Function timeBeginPeriod(ByVal uPeriod As Int32) As Int32
    End Function
    <System.Runtime.InteropServices.DllImport("winmm.dll")> _
    Private Shared Function timeEndPeriod(ByVal uPeriod As Int32) As Int32
    End Function
    Private Sub MillisecondPrecision()
        timeBeginPeriod(1)
    End Sub
    Private Sub NormalPrecision()
        timeEndPeriod(1)
    End Sub


    ' ===============================================================
    '   USER INTERFACE TIMER
    ' =============================================================== 
    Private Sub UserInterfaceTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles UserInterfaceTimer.Tick
        Static oldoperation As Int32 = -1
        If Operation <> oldoperation Then
            oldoperation = Operation
            ToolStripStatusLabel1.Text = OperationName(Operation)
            Select Case Operation
                Case 0
                    ToolStripButton_Start.Enabled = True
                    ToolStripButton_Stop.Enabled = False
                    ToolStripButton_WriteDataFile.Enabled = True
                    GroupBox_SurveyParams.Enabled = True
                    GroupBox_HardwareParams.Enabled = True
                    GroupBox_AcquisitionParams.Enabled = True
                    GroupBox_ViewParams.Enabled = True
                    TabPage2.Enabled = True
                Case 1, 2
                    ToolStripButton_Start.Enabled = False
                    ToolStripButton_Stop.Enabled = True
                    ToolStripButton_WriteDataFile.Enabled = False
                    GroupBox_SurveyParams.Enabled = False
                    GroupBox_HardwareParams.Enabled = False
                    GroupBox_AcquisitionParams.Enabled = False
                    GroupBox_ViewParams.Enabled = True
                    TabPage2.Enabled = False
            End Select
        End If

        Static oldpercentual As Int32
        If Percentual <> oldpercentual Then
            If Percentual > 100 Then Percentual = 100
            oldpercentual = Percentual
            ToolStripProgressBar1.Value = Percentual
        End If

        If UpdatingGraph Then
            UpdateGraph()
        End If
    End Sub

    Private Function OperationName(ByVal operation As Int32) As String
        Select Case operation
            Case 0 : Return ""
            Case 1 : Return Msg_OperationZeroSetting
            Case 2 : Return Msg_OperationAcquisition
            Case Else : Return ""
        End Select
    End Function

    Private Sub UpdateGraph()
        ' --------------------------------------------------------------------- 
        Dim w As Int32 = PictureBox1.Image.Width
        Dim h As Int32 = PictureBox1.Image.Height
        ' ---------------------------------------------------------------------
        If x * ZoomX > w Then x = 0
        ' --------------------------------------------------------------------- 
        Dim x1 As Int32 = (x - 1) * ZoomX
        Dim x2 As Int32 = x * ZoomX
        Dim y As Int32
        ' --------------------------------------------------------------------- Yscale variable with gain
        Dim ky As Single = 10000.0F * ZoomY / VoltageGain

        Dim value As Single
        ' --------------------------------------------------------------------- BACKGROUND CLEARING
        GFX.FillRectangle(New SolidBrush(PictureBox1.BackColor), x1 + 1, 0, x2 + 10, h)
        ' --------------------------------------------------------------------- 
        For i As Int32 = 1 To NumChannels
            value = LastValue(i) * ky
            y = CInt(i * (h / (NumChannels + 1)))
            GFX.DrawLine(PenColors(i), x1, y + OldValue(i), x2, y + value)
            If LastValue(i) < 0 Then
                GFX.DrawLine(PenColors(i), x2, y, x2, y + value)
            End If
            OldValue(i) = value
        Next
        x += 1
        ' --------------------------------------------------------------------- PICTUREBOX UPDATE
        Dim rec As Rectangle
        rec.X = x1
        rec.Width = ZoomX
        rec.Height = h
        PictureBox1.Invalidate(rec)  ' invalidate only the modified rectangle
    End Sub

    ' =======================================================================================
    '   VisualizationFreq not faster then SamplingFreq and not more than 100
    ' =======================================================================================
    Private Sub cmb_VisualizationFreq_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmb_VisualizationFreq.SelectedIndexChanged
        CorrectVisualizationFreq()
    End Sub
    Private Sub cmb_SamplingFreq_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmb_SamplingFreq.SelectedIndexChanged
        CorrectVisualizationFreq()
    End Sub
    Private Sub CorrectVisualizationFreq()
        If cmb_VisualizationFreq.SelectedIndex < cmb_SamplingFreq.SelectedIndex - 5 Then
            cmb_VisualizationFreq.SelectedIndex = cmb_SamplingFreq.SelectedIndex - 5
        End If
        If cmb_VisualizationFreq.SelectedIndex > 3 Then
            lbl_VisualizationFreqNote.ForeColor = Color.DimGray
        ElseIf cmb_VisualizationFreq.SelectedIndex > 1 Then
            lbl_VisualizationFreqNote.ForeColor = Color.DarkRed
        Else
            lbl_VisualizationFreqNote.ForeColor = Color.Red
        End If
    End Sub

    ' =======================================================================================
    '   Visualization params update while running
    ' =======================================================================================
    Private Sub VisualizationParams_Update(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmb_VisualizationFreq.SelectedIndexChanged, _
                                                                                                               cmb_ZoomX.SelectedIndexChanged, _
                                                                                                               cmb_ZoomY.SelectedIndexChanged
        If Not EventsAreEnabled Then Exit Sub
        SetVisualizationParams()
    End Sub

    ' ===============================================================
    '   SET PARAMS 
    ' =============================================================== 
    Private Sub SetSurveyParams()
        AcquisitionSystem = "Theremino Dolfrang"
        SurveyName = txt_SurveyName.Text
        Locality = txt_Locality.Text
        Department = txt_Department.Text
        Notes = txt_Notes.Text
    End Sub
    Private Sub SetHardwareParams()
        Rotation = txt_Rotation.Text
        StationCode = txt_StationCode.Text
        VoltageGain = CInt(Val(txt_VoltageGain.Text))
        If VoltageGain < 1 Then VoltageGain = 1 : txt_VoltageGain.Text = "1"
        OutDataFormat = cmb_OutDataFormat.Text
        TransducerType = cmb_TransducerType.Text
        IdChannel1 = cmb_Channel1.Text
        IdChannel2 = cmb_Channel2.Text
        IdChannel3 = cmb_Channel3.Text
    End Sub
    Private Sub SetAcquisitionParams()
        SamplingFreq_hz = CInt(Val(cmb_SamplingFreq.Text.Replace(",", ".")))
        If SamplingFreq_hz < 1 Then SamplingFreq_hz = 1
        If SamplingFreq_hz > 500 Then SamplingFreq_hz = 500
        SamplingTime_ms = CInt(1000 / SamplingFreq_hz)
        If SamplingTime_ms < 2 Then SamplingTime_ms = 2
        ' --------------------------------------------------------------------- Num channels
        NumChannels = CInt(Val(cmb_NumChannels.Text.Replace(",", ".")))
        ' --------------------------------------------------------------------- Zeroing time
        ZeroingTime_ms = 1000 * CInt(Val(cmb_ZeroingTime.Text))
        ' --------------------------------------------------------------------- Acquisition time
        AcquisitionTime_ms = 1000 * CInt(Val(cmb_AcquisitionTime.Text))
        If cmb_AcquisitionTimeUnit.SelectedIndex = 1 Then
            AcquisitionTime_ms *= 60
        End If
    End Sub
    Private Sub SetVisualizationParams()
        ' --------------------------------------------------------------------- Visualization frequency and time
        VisualizationFreq_hz = CInt(Val(cmb_VisualizationFreq.Text.Replace(",", ".")))
        If VisualizationFreq_hz < 1 Then VisualizationFreq_hz = 1
        If VisualizationFreq_hz > 500 Then VisualizationFreq_hz = 500
        VisualizationTime_ms = CInt(1000 / VisualizationFreq_hz)
        If VisualizationTime_ms < 2 Then VisualizationTime_ms = 2
        UserInterfaceTimer.Interval = VisualizationTime_ms
        ' --------------------------------------------------------------------- Visualization Zoom
        ZoomX = CInt(Val(cmb_ZoomX.Text))
        ZoomY = CInt(Val(cmb_ZoomY.Text))
    End Sub


    ' ===============================================================
    '   START ACQUISITION
    ' ===============================================================       
    Private Sub StartAcquisition()
        ' --------------------------------------------------------------------- get all params
        WriteDataFile = ToolStripButton_WriteDataFile.Checked
        SetSurveyParams()
        SetHardwareParams()
        SetAcquisitionParams()
        SetVisualizationParams()
        SetChannelGains()
        ' --------------------------------------------------------------------- 
        InitPictureboxImage()
        ' --------------------------------------------------------------------- start acquisition thread
        If AcquisitionThread Is Nothing OrElse Not AcquisitionThread.IsAlive Then
            AcquisitionThread = New Thread(AddressOf ZeroingAndAcquisition)
            AcquisitionThread.IsBackground = True
            AcquisitionThread.Priority = ThreadPriority.Highest
            AcquisitionThread.Start()
        End If
    End Sub

    Private Sub ZeroingAndAcquisition()
        ' ---------------------------------------------------------------------
        Dim TotalSamples As Int32 = AcquisitionTime_ms \ SamplingTime_ms
        Dim ThisTime As Double
        Dim NextTime As Double
        Dim i As Int32
        ' --------------------------------------------------------------------- ZEROING
        MillisecondPrecision()
        AcquisitionTimer.Reset()
        AcquisitionTimer.Start()
        Operation = 1
        StopAcquisitionFlag = False
        Beep()
        ReadSamplesTotal = 0
        TotalSamples = CInt(ZeroingTime_ms / SamplingTime_ms)
        ThisTime = 0
        NextTime = 0
        ' --------------------------------------------------------------------- RESET MEDIA
        For i = 1 To NumChannels
            MidValue(i) = 0
        Next
        ' --------------------------------------------------------------------- VALUES ACCUMULATION
        If ZeroingTime_ms > 0 Then
            AcquisitionTimer.Reset()
            AcquisitionTimer.Start()
            Do
                Percentual = CInt(ThisTime * 100 / ZeroingTime_ms)
                ReadSamplesTotal += 1
                '
                For i = 1 To NumChannels
                    MidValue(i) += Slots.ReadSlot_NoNan(i)
                Next
                '
                NextTime += SamplingTime_ms
                Do
                    Threading.Thread.Sleep(1)
                    ThisTime = AcquisitionTimer.Elapsed.TotalMilliseconds
                Loop Until ThisTime >= NextTime
                '
            Loop Until ReadSamplesTotal >= TotalSamples Or StopAcquisitionFlag
            AcquisitionTimer.Stop()
            Percentual = 0
            ' --------------------------------------------------------------------- MID VALUE CALCULATION
            For i = 1 To NumChannels
                MidValue(i) = MidValue(i) / ReadSamplesTotal
            Next
        End If
        ' --------------------------------------------------------------------- ACQUISITION
        Beep()
        Operation = 2
        TotalSamples = CInt(AcquisitionTime_ms / SamplingTime_ms)
        ' --------------------------------------------------------------------- OPEN THE OUTPUT FILE
        Dim Path As String = Application.StartupPath & "\DataFiles\"
        'Dim Name As String = "Test1.saf"
        Dim Name As String = txt_SurveyName.Text & "_" & _
                             DateTime.Now.Year & "_" & _
                             DateTime.Now.Month & "_" & _
                             DateTime.Now.Day & "_" & _
                             DateTime.Now.Hour & "_" & _
                             DateTime.Now.Minute & ".saf"

        Directory.CreateDirectory(Path)
        ' --------------------------------------------------------------------- SAF PARAMS
        Dim File As StreamWriter = Nothing
        Try
            ' START_TIME=2002 4 26 17 03 12.1233324 (5 integers, 1 real)
            If WriteDataFile Then
                File = IO.File.CreateText(Path & Name)
                File.WriteLine("SESAME ASCII data format (saf) v. 1")
                File.WriteLine("# ----------------------------------")
                File.WriteLine("START_TIME  = " & _
                               DateTime.Now.Year & " " & _
                               DateTime.Now.Month & " " & _
                               DateTime.Now.Day & " " & _
                               DateTime.Now.Hour & " " & _
                               DateTime.Now.Minute & " " & _
                               DateTime.Now.Second & "." & _
                               DateTime.Now.Millisecond)
                File.WriteLine("ACQ_SYSTEM  = " & AcquisitionSystem)
                File.WriteLine("SURVEY_NAME = " & SurveyName)
                If Department <> "" Then File.WriteLine("Department  = " & Department)
                If Locality <> "" Then File.WriteLine("Locality    = " & Locality)
                If Notes <> "" Then File.WriteLine("Notes       = " & Notes)
                File.WriteLine("STA_CODE    = " & StationCode)
                File.WriteLine("SAMP_FREQ   = " & SamplingFreq_hz)
                File.WriteLine("NDAT        = " & TotalSamples)
                File.WriteLine("NCHAN       = " & NumChannels)
                File.WriteLine("NORTH_ROT   = " & Rotation)
                File.WriteLine("SENSOR_TYPE = " & TransducerType)
                File.WriteLine("UNITS       = " & OutDataFormat)
                File.WriteLine("CH0_ID      = " & IdChannel1)
                File.WriteLine("CH1_ID      = " & IdChannel2)
                File.WriteLine("CH2_ID      = " & IdChannel3)
                File.WriteLine("####--------------------------------")
            End If

            ' --------------------------------------------------------------------- 
            Dim FormatString As String = "+0.000000E+0;-0.000000E+0"
            x = 0
            UpdatingGraph = True
            MillisecondPrecision()
            ReadSamplesTotal = 0
            ThisTime = 0
            NextTime = 0
            AcquisitionTimer.Reset()
            AcquisitionTimer.Start()
            ' --------------------------------------------------------------- do the acquisition loop
            Do
                Percentual = CInt(ThisTime * 100 / AcquisitionTime_ms)
                ReadSamplesTotal += 1
                '
                For i = 1 To NumChannels
                    LastValue(i) = Slots.ReadSlot_NoNan(i) - MidValue(i)
                    If WriteDataFile Then
                        File.Write((K(i) * LastValue(i)).ToString(FormatString, GCI) & vbTab)
                    End If
                Next
                If WriteDataFile Then File.Write(File.NewLine)
                '
                NextTime += SamplingTime_ms
                Do
                    Threading.Thread.Sleep(1)
                    ThisTime = AcquisitionTimer.Elapsed.TotalMilliseconds
                Loop Until ThisTime >= NextTime
                '
            Loop Until ReadSamplesTotal >= TotalSamples Or StopAcquisitionFlag
            ' --------------------------------------------------------------- 
            AcquisitionTimer.Stop()
            UpdatingGraph = False
            ' --------------------------------------------------------------- 
            If StopAcquisitionFlag Then
                StopAcquisitionFlag = False
                If WriteDataFile Then
                    File.Write("# " & Msg_AcquInterrupted)
                End If
                MsgBox(Msg_AcquInterrupted, MsgBoxStyle.SystemModal Or MsgBoxStyle.Information)
            Else
                Dim ElapsedMillisec As Double = AcquisitionTimer.ElapsedMilliseconds / 1000
                MsgBox(Msg_Time & ": " & ElapsedMillisec.ToString() & vbCr & vbCr & _
                       Msg_Samples & ": " & ReadSamplesTotal.ToString() & vbCr & vbCr & _
                       Msg_Frequency & ": " & (ReadSamplesTotal / ElapsedMillisec).ToString & _
                       "                  ", MsgBoxStyle.SystemModal Or MsgBoxStyle.Information)
            End If
            ' --------------------------------------------------------------- 
            Operation = 0
        Catch
            MsgBox(Msg_OutputFileError, MsgBoxStyle.Critical)
        End Try
        Try
            If WriteDataFile Then
                File.Close()
            End If
        Catch
        End Try
        ' ---------------------------------------------------------------- to be sure in case of error catch
        AcquisitionTimer.Stop()
        NormalPrecision()
        Percentual = 0
        Operation = 0
        UpdatingGraph = False
    End Sub


    ' ================================================================
    '   Set the K ARRAY with gains for all the channels from 1 to 24
    ' ================================================================ 
    Private kVoltage As Single
    Friend K_HV As Single = 1
    Friend K_NE As Single = 1
    Private K(24) As Single

    Private Sub SetChannelGains()
        ' --------------------------------------------------------------------- Voltage Coefficient
        ' ............................................................
        '  Starting from version 5.3 values from slots are in "Volt"
        '  With Adc24 set min/max = -3.3/+3.3
        '  With Adc16 set min/max =   0 /+3.3
        ' ............................................................
        Select Case OutDataFormat
            Case "mV"
                kVoltage = 1000.0F / VoltageGain
            Case "uV"
                kVoltage = 1000000.0F / VoltageGain
        End Select
        ' --------------------------------------------------------------------- K for all the channels
        For i As Int32 = 0 To 24
            K(i) = kVoltage
        Next
        ' --------------------------------------------------------------------- HV trimming coefficients
        K(1) *= GetHvTrimmingCoefficient(IdChannel1)
        K(2) *= GetHvTrimmingCoefficient(IdChannel2)
        K(3) *= GetHvTrimmingCoefficient(IdChannel3)
    End Sub

    ' ------------------------------------------------------------------------- H/V and breeze coefficients
    Private Function GetHvTrimmingCoefficient(ByVal IdChannel As String) As Single
        Dim KV As Single = 1
        Dim KN As Single
        Dim KE As Single
        ' -------------------------------------------
        KN = 1 / CSng(Val(cmb_BreezeAttenuation.Text))
        KN *= K_HV
        KE = KN
        KN *= K_NE
        KE /= K_NE
        ' -------------------------------------------
        Select Case IdChannel
            Case "V" : Return KV
            Case "N" : Return KN
            Case "E" : Return KE
            Case Else : Return 1
        End Select
    End Function

    Private Sub cmb_HVScale_DropDownClosed(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmb_HVScale.DropDownClosed
        K_HV *= CSng(Val(cmb_HVScale.Text))
        ShowKvalues()
        cmb_HVScale.Text = "1"
    End Sub
    Private Sub cmb_NEScale_DropDownClosed(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmb_NEScale.DropDownClosed
        K_NE *= CSng(Val(cmb_NEScale.Text))
        ShowKvalues()
        cmb_NEScale.Text = "1"
    End Sub

    Private Sub ShowKvalues()
        lbl_TrimHV.Text = "K = " + K_HV.ToString("0.000", GCI)
        lbl_TrimNE.Text = "K = " + K_NE.ToString("0.000", GCI)
    End Sub

End Class








